/*
 * =====================================================================================
 * 
 *        Filename:  Image.cc
 * 
 *     Description:  
 * 
 *         Version:  1.0
 *         Created:  05/10/05 15:04:46 CEST
 *        Revision:  none
 *        Compiler:  gcc
 * 
 *          Author:  Janis Fehr (JF), fehr@informatiik.uni-freiburg.de
 *         Company:  LMB
 * 
 * =====================================================================================
 */

#include "Image.hh"


/*======================================================================*/
/*!
 *   eats white spaces and comments (starting with '#'
 *   until line end) from the given stream
 *
 *   \param  s  stream
 *
 *======================================================================*/
std::istream& eatWsAndComments(std::istream &s)
{
  s >> std::ws;

  while (s.peek()=='#')
  {
    s.ignore(10000, '\n');  // ignore characters until line end
    s >> std::ws;
  }
  return s;
}


template<>
void Image<RGBColor>::write(std::ostream& os)
{
  os << "P3\n"
     << width() << " " << height() << "\n"
     << "255\n";

  for (const RGBColor* p=begin();
       p!=end();
       ++p)
  {
    os << (unsigned int)p->red() << "\n"
       << (unsigned int)p->green() << "\n"
       << (unsigned int)p->blue() << "\n" ;
  }

}

template<>
void Image<RGBColor>::read(std::istream& is)
{
  std::string header;
  is >> std::setw(2+1) >> header;
  if (!is.good())
  {
    MyError e("Error while reading P3");
    throw e;
  }

  if (header!="P3")
  {
    MyError e("Unknown header " + header + ", expected P3");
    throw e;
  }

  /*-------------------------------------------------------------------
   *  read width, height and maximum color-component value
   *-------------------------------------------------------------------*/
  int width;
  is >> eatWsAndComments >> width;
  if (!is.good())
  {
    throw MyError("error while reading image width");
  }

  int height;
  is >> eatWsAndComments >> height;
  if (!is.good())
  {
    throw MyError("error while reading image height\n");
  }

  int maxVal;
  is >> eatWsAndComments >> maxVal;
  if (!is.good() || maxVal>255)
  {
    throw MyError("only maxVal<=255 supported");
  }

  /*-------------------------------------------------------------------
   *  resize memory for image if size changed
   *-------------------------------------------------------------------*/
  resize(width, height);

  for (RGBColor* p=begin();
       p!=end();
       ++p)
  {
    if (!is.good())
    {
      throw MyError("error while reading image pixels");
    }

    unsigned int r, g, b;
    is >> r >> g >> b;

    if (r>255 || g>255 || b>255)
    {
      throw MyError("color channel value to large");
    }

    p->set(r, g, b);
  }

  //is >> ws;  // eat trailing whitespaces, if there are any
  is >> eatWsAndComments;
}


